@@ -5,11 +5,11 @@ use strict;
 use Carp;
 use UNIVERSAL qw/isa/;
 use File::Spec;
-use File::BaseDir qw/xdg_config_files xdg_data_files/;
+use File::BaseDir qw/xdg_config_files xdg_data_home xdg_data_dirs xdg_data_files/;
 use Zim::Selection;
 use Zim::File;
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 =head1 NAME
 
@@ -97,6 +97,21 @@ Returns a hash with config options.
 
 sub config { $_[0]->{config} }
 
+=item C<save_config()>
+
+Saves the config to disk.
+
+=cut
+
+sub save_config {
+	my $self = shift;
+	my $node = $self->{dir};
+	my $conf = Zim::File->new($node, '.zim.config');
+	my $text = join '', map "$_=$$self{config}{$_}\n", keys %{$$self{config}};
+	for ($conf->read) { $text .= $_ if /^:/ }
+	$conf->write($text);
+}
+
 =item C<base()>
 
 Returns the base directory for files.
@@ -134,6 +149,10 @@ sub _check_namespace {
 		unless $_[0] =~ /^(?::+|(?::+\w[\w\.\-\(\)]*)+:+)$/;
 }
 
+=item C<has_pages(NAMESPACE)>
+
+Returns boolean whether NAMESPACE exists.
+
 =item C<list_pages(NAMESPACE)>
 
 Lists pages in NAMESPACE. Sub-namespaces have a trailing ':'
@@ -145,12 +164,27 @@ page structure; not all repositories need to support a hierarchical order.
 
 =cut
 
-sub list_pages { # TODO also list plugin namespaces + parents
+sub has_pages {
 	my ($self, $namespace) = @_;
 	$namespace =~ s/:?$/:/;
 	_check_namespace($namespace);
-	$namespace =~ $self->{regex};
-	return $self->{":$1"}->list_pages($namespace);
+
+	return 1 if grep /^$namespace/i, keys %$self;
+
+	$namespace =~ $self->{regex} or die "BUG: '$namespace' !~ qr/$self->{regex}/";
+	$self->{lc($1)}->has_pages($namespace);
+}
+
+sub list_pages {
+	my ($self, $namespace) = @_;
+	$namespace =~ s/:?$/:/;
+	_check_namespace($namespace);
+	my @mount_points =
+		grep { m/^:/ and s/$namespace:*([^:]+:*)$/$1:/ } keys %$self;
+	$namespace =~ $self->{regex} or die "BUG: '$namespace' !~ qr/$self->{regex}/";
+	return if $self->{lc($1)}->{no_show_in_sidepane}; # temp HACK
+	return sort {lc($a) cmp lc($b)}
+		@mount_points, $self->{lc($1)}->list_pages($namespace);
 }
 
 =item get_page(NAME)
@@ -164,9 +198,10 @@ In case it is not possible to create the page in question C<undef> is returned.
 sub get_page {
 	my ($self, $name) = @_;
 	_check_page_name($name);
-	$name =~ $self->{regex};
-	#warn "Dispatching get_page('$name') to ':$1'\n";
-	return $self->{":$1"}->get_page($name);
+	$name =~ $self->{regex} or die "BUG: '$name' !~ qr/$self->{regex}/";
+	#warn "'$name' =~ /$self->{regex}/ => \$1 = '$1'\n";
+	#warn "Dispatching get_page('$name') to '".lc($1)."'\n";
+	return $self->{lc($1)}->get_page($name);
 }
 
 =item C<resolve_page(NAME)>
@@ -185,9 +220,10 @@ sub resolve_page {
 	$name =~ s/(:+)[\_\.\-\(\)]+/$1/g;	# remove non-letter at begin
 	$name =~ s/_+(:|$)/$1/g;		# remove trailing underscore
 	#print STDERR "$name\n";
-	$name =~ $self->{regex};
-	#warn "Dispatching resolve_page('$name') to ':$1'\n";
-	return $self->{":$1"}->resolve_page($name);
+	$name =~ $self->{regex} or die "BUG: '$name' !~ qr/$self->{regex}/";
+	#warn "Dispatching resolve_page('$name') to '".lc($1)."'\n";
+	#warn "Regex was: $self->{regex}\n";
+	return $self->{lc($1)}->resolve_page($name);
 }
 
 =item C<resolve_namespace(NAME)>
@@ -232,7 +268,9 @@ sub _clone { # Copy and move are almost the same
 	$target = $self->get_page($target) || die "Could not create page '$target'\n"
 	       	unless ref $target;
 	
-	die "Page '$target' exists\n" if $target->exists;
+	die "Page '$target' exists\n"
+		if $target->exists and ! $target->equals($source);
+		# if pages are equal the case can still differ
 	
 	if ($source->{repository} eq $target->{repository}) {
 		$source->{repository}->$method($source, $target);
@@ -260,9 +298,11 @@ sub delete_page {
 	return $page;
 }
 
-=item C<search(PAGE, QUERY)>
+=item C<search(QUERY, CALLBACK)>
 
-TODO stable api for this
+TODO stable api for this using Zim::Selection
+
+Results are given as arguments to CALLBACK in the form C<[PAGE, SCORE]>.
 
 =cut
 
@@ -270,11 +310,11 @@ TODO stable api for this
 # on subsets of pages  and combines backlink lists with other keywords
 
 sub search {
-	my ($self, $query) = @_;
-	return map {
-		$self->{$_}->can('_search')
-			? ( $self->{$_}->_search($query) ) : ()
-	} grep /^:/, keys %$self;
+	my ($self, $query, $callback) = @_;
+	for (grep /^:/, keys %$self) {
+		$self->{$_}->_search($query, $callback)
+			if $self->{$_}->can('_search');
+	}
 }
 
 =item C<export(PAGE, %OPTS)>
@@ -298,7 +338,16 @@ sub export {
 	my $exporter = ref($opts{root}) ? $opts{root} :
 		Zim::Repository::Files->new(undef, '', $opts{root}, $opts{format});
 	#print STDERR "export template: $opts{template}\n";
-	$exporter->{_template} = $opts{template}; # FIXME better interface for this - do not confuse with the template for new pages
+	my $template = $opts{template};
+	unless (-f $template) {
+		if ($template =~ m#/#) { die "No such file: $template\n" }
+		else {
+			my $key = $template;
+			$template = Zim::Repository->lookup_template($opts{format}, $key);
+			die "No such template: $key" unless defined $template;
+		}
+	}
+	$exporter->{_template} = $template; # FIXME better interface for this - do not confuse with the template for new pages
 	
 	$opts{recurs} = 1 unless defined $opts{recurs};
 	if (length $namespace) {
@@ -442,43 +491,49 @@ sub add_child {
 	my ($self, $namespace, $child, @args) = @_;
 	$namespace =~ s/:?$/:/;
 	_check_namespace($namespace);
-	#warn "Adding child  type '$child' for $namespace @args\n";
+	#warn "Adding child  type '$child' for '$namespace' (@args)\n";
 	
 	$child = "Zim::Repository::$child" unless $child =~ /::/;
 	eval "use $child ();";
 	die if $@;
 	$child = $child->new($self, $namespace, @args);
-	$self->{$namespace} = $child;
 
+	$namespace =~ s/:*$//;
+	$namespace = ':' if $namespace eq '';
+	$self->{lc($namespace)} = $child;
+
+	my $ns = $self->{namespace};
+	$ns =~ s/:*$//;
 	my $regex = join '|', reverse
-		sort {length($a) <=> length($b)} grep s/^://, keys %$self;
-	$self->{regex} = qr#$self->{namespace}(?i)($regex)#;
+		sort {length($a) <=> length($b)} grep /^:./, keys %$self;
+	$regex = length($regex) ? "(?:$regex)(?![^:])|:" : ':';
+	$self->{regex} = qr#^(?i)$ns($regex)#;
 	#warn "regex: $self->{regex}\n";
 }
 
 =item C<interwiki_lookup(KEY, PAGE)>
 
+Returns an url for an interwiki link with interwiki name KEY and page PAGE.
+Lookup for names is case in-sensitive.
+
 =cut
 
 our %_urls;
 
 sub interwiki_lookup { # FIXME linked from Repository base for export stuff
 	my ($self, $key, $page) = @_;
-
+	$key = lc $key;
 	my $url;
 	if (exists $_urls{$key}) { $url = $_urls{$key} }
 	else { # file lookup
 		# TODO repository specific urls here
 		
 		# Lookup named zim repositories
-		for (xdg_config_files('zim', 'repositories.list')) {
-			my ($hash) = Zim::File->new($_)->read_config;
-			next unless exists $$hash{$key};
-			my $file = Zim::File->abs_path($$hash{$key}, $ENV{HOME});
+		my $file = Zim::Repository->lookup_by_name($key);
+		if (defined $file) {
 			$url = Zim::File->path2uri($file);
 			$url =~ s#^file:#zim:#i;
 			$url .= '?{NAME}';
-			last;
 		}
 
 		# lookup interwiki config
@@ -490,7 +545,7 @@ sub interwiki_lookup { # FIXME linked from Repository base for export stuff
 		}
 	}
 	return unless length $url;
-	$_urls{$key} = $url;
+	$_urls{lc($key)} = $url;
 
 	unless ($url =~ s/{NAME}/$page/) {
 		$page =~ s/([^A-Za-z0-9\-_.!~*'()])/sprintf("%%%02X", ord($1))/ge;
@@ -500,6 +555,50 @@ sub interwiki_lookup { # FIXME linked from Repository base for export stuff
 	return $url;
 }
 
+=item C<relative_path(FILE, PAGE)>
+
+Checks to see if the path for FILE can be made relative to the path for PAGE.
+If PAGE is undef tries to make relative to the repository.
+Returns either the relative or the absolute path.
+
+=cut
+
+sub relative_path {
+	# 3 way directory comparison
+	# Returns path relative to 'ref' when file is below 'root'
+	# returns absolute filename otherwise
+	my ($self, $file, $page) = @_;
+	my $ref = $page ? $page->properties->{base} : $self->{dir};
+	$ref = Zim::File->abs_path($ref);
+	return $file unless defined $ref;
+	$file = Zim::File->abs_path($file,$ref);
+	return $file if $file =~ m#^\w\w+:/#;
+	my $root = $self->{dir};
+	$root = Zim::File->abs_path($root);
+	#warn "check whether '$file' is below '$ref'\n";
+
+	my @root = split m#/+#, $root;
+	my @ref = split m#/+#, $ref;
+	my @file = split m#/+#, $file;
+	for (@root) { # remove root part - return if no match
+		my $dir = shift @file;
+		#warn "match $dir $_\n";
+		return $file if $dir ne $_;
+		$dir = shift @ref;
+		return $file if $dir ne $_; # just to be sure
+	}
+	while (@ref) { # remove common path
+		last if $ref[0] ne $file[0];
+		shift @ref;
+		shift @file;
+	}
+
+	if (@ref) { unshift @file, '..' for 0 .. $#ref }
+	else      { unshift @file, '.'                 }
+	
+	return join '/', @file; # un-localize
+}
+
 =item C<split_page(PAGE, LEVEL)>
 
 Splits a page into multiple pages in the like-named namespace.
@@ -627,6 +726,81 @@ sub _flush_cache {
 	}
 }
 
+=back
+
+=head2 Class Methods
+
+These methods can be called as C<< Zim::Repository->method(...) >>.
+
+=over 4
+
+=item C<lookup_by_name(NAME)>
+
+Returns a path for a repository with NAME or undef if the repository is unknown.
+The lookup is case in-sensitive.
+
+=cut
+
+sub lookup_by_name {
+	my ($class, $name) = @_;
+	$name = lc $name;
+	for (xdg_config_files('zim', 'repositories.list')) {
+		my ($hash) = Zim::File->new($_)->read_config;
+		my ($key) = grep {lc($_) eq $name} keys %$hash;
+		return Zim::File->abs_path($$hash{$key}, $ENV{HOME})
+			if defined $key;
+	}
+	return undef;
+}
+
+=item C<lookup_template(FORMATE, NAME)> 
+
+Returns a file name or undef.
+
+TODO: This method should go to either Zim::Template or Zim::Formats
+when these are created.
+
+=cut
+
+sub lookup_template {
+	my ($class, $format, $name) = @_;
+	my $map = $class->list_templates($format);
+	return $$map{$name} if exists $$map{$name};
+	my ($key) = grep {lc $_ eq lc $name} keys %$map;
+	return $$map{$key} if defined $key;
+	return undef;
+}
+
+=item C<list_templates(FORMAT)>
+
+Returns a hash with name => filename pairs of available templates.
+Used for exporting.
+
+TODO: This method should go to either Zim::Template or Zim::Formats
+when these are created.
+
+=cut
+
+sub list_templates {
+	my ($class, $format) = @_;
+	die "FIXME: only html format supported" unless $format eq 'html';
+	# Either have a directory with templates for each format
+	# or have a format => extension map somewhere
+
+	my %templates;
+	for (xdg_data_home(), xdg_data_dirs()) {
+		my $dir = File::Spec->catdir($_, 'zim', 'templates');
+		next unless -d $dir;
+		for my $f (Zim::File->list_dir($dir)) {
+			next unless $f =~ s/\.html$//;
+			$templates{$f} = "$dir/$f.html"
+				unless defined $templates{$f};
+		}
+	}
+
+	return \%templates;
+}
+
 1;
 
 __END__
