@@ -5,7 +5,7 @@ use File::Spec;
 use Zim::Repository::Base;
 use Zim::File;
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 our @ISA     = 'Zim::Repository::Base';
 
 my $null = File::Spec->devnull;
@@ -33,6 +33,7 @@ sub init {
 	my $path = join '', <MAN>;
 	close MAN;
 	$self->{path} = [grep length($_), split /:+/, $path];
+	$self->{no_show_in_sidepane} = 1; # temp HACK
 	#warn "MANPATH: @{$self->{path}}\n";
 }
 
@@ -87,42 +88,14 @@ sub get_page {
 	my ($self, $name) = @_;
 	#warn "Get man page: $name\n";
 	
-	$name =~ s/^$self->{namespace}:*//;
-	$name =~ s/^(\d+\w*):+//;
+	my $n = $name;
+	$n =~ s/^$self->{namespace}:*//;
+	$n =~ s/^(\d+\w*):+//;
 	my $sect = $1 || '';
 	
-	$ENV{MANWIDTH} = 80; # FIXME get window size (via Env ?)
-	open MAN, "man -c $sect $name 2> $null |" or return undef;
-	my ($block, @data);
-	while (<MAN>) {
-		# FIXME implement parsing algo like in Zim.pm
-		# include bold and head2
-		#s/((\S\cH\S)+)/<b>$1<\/b>/g;
-		chomp;
-		s/.\cH//g;
-		if (/^[A-Z]+[A-Z\s]*$/) { # heading
-			push @data, $block if length $block;
-			push @data, ['head1', {}, $_];
-			$block = '';
-		}
-		elsif (/\b[\w\:\.\-]+\(\w+\)/) { # links
-			# FIXME namespace links per man section
-			push @data, $block if length $block;
-			while (s/(.*?)\b([\w\:\.\-]+\(\d+\w*\))//) {
-				push @data, $1 if length($1);
-				push @data, ['link', {to => $2}, $2];
-			}
-			$block = $_ . "\n";
-		}
-		else { $block .= $_ . "\n" }
-	}
-	push @data, $block if length $block;
-	close MAN;
-	return undef unless @data;
-	
-	$name = $self->{namespace}.($sect ? $sect.':' : '').$name;
 	my $page = Zim::Man::Page->new($self, $name);
-	$page->set_parse_tree(['Document', {}, @data]);
+	$page->{format} = '_man'; # to force formatted interface
+	$page->{_manpage} = [$n, $sect];
 	
 	return $page;
 }
@@ -172,6 +145,57 @@ use Zim::Page::Text;
 
 our @ISA = qw/Zim::Page::Text/;
 
+=item C<get_parse_tree()>
+
+Returns parse tree for man page.
+
+=cut
+
+sub get_parse_tree {
+	my ($self) = @_;
+	return $self->{parse_tree} if defined $self->{parse_tree};
+	my ($name, $sect) = @{$self->{_manpage}};
+	
+	$ENV{MANWIDTH} = 80; # FIXME get window size (via Env ?)
+	open MAN, "man -c $sect $name 2> $null |" or return undef;
+	my ($block, @data);
+	while (<MAN>) {
+		# FIXME implement parsing algo like in Zim.pm
+		# include bold and head2
+		#s/((\S\cH\S)+)/<b>$1<\/b>/g;
+		chomp;
+		s/.\cH//g;
+		if (/^[A-Z]+[A-Z\s]*$/) { # heading
+			push @data, $block if length $block;
+			push @data, ['head1', {}, $_];
+			$block = '';
+		}
+		elsif (/\b[\w\:\.\-]+\(\w+\)/) { # links
+			# FIXME namespace links per man section
+			push @data, $block if length $block;
+			while (s/(.*?)\b([\w\:\.\-]+\(\d+\w*\))//) {
+				push @data, $1 if length($1);
+				push @data, ['link', {to => $2}, $2];
+			}
+			$block = $_ . "\n";
+		}
+		else { $block .= $_ . "\n" }
+	}
+	push @data, $block if length $block;
+	close MAN;
+	
+	$self->{parse_tree} = ['Document', {}, @data];
+	return $self->{parse_tree};
+}
+
+=item C<set_parse_tree(TREE)>
+
+Man pages can notbe saved, thus it throws away all data.
+
+=cut
+
+sub set_parse_tree { warn "Man pages are not writable.\n" }
+
 =item C<parse_link(LINK)>
 
 Overload default parsing for man page links.
